#!/bin/sh

echo "+++++ Checking current user rights..."

if [ "$(id -u root)" = "$(id -u)" ] ; then
    :
else
    echo "ERROR: The current user id is not root (`id|awk '{print $1}'`)."
    exit 1
fi
echo

if [ "X$PVSW_ROOT" = "X" ] ; then
    PVSW_ROOT=/usr/local/psql
    export PVSW_ROOT=$PVSW_ROOT
fi

# Determine if system libstdc meets Zen version requirements and remove
# bundled version if it does.
if [ $(uname -s) != "Darwin" ]; then
    echo "+++++ Checking system libstdc++ version support..."
    # Verify readelf is available.
    READELF=`which readelf`
    if [ ! -f "$READELF" ]; then
        echo "WARNING: Unable to determine system libstdc++ version support (missing readelf command)."
        echo "The libstdc++ libraries bundled with the Zen install will be used, which may prevent the"
        echo "opening of documentation from within ZenCC and possibly other issues running Zen utilities."
    else
        ZEN_REQUIRED_LIBSTDC_VER=3.4.21
        ZEN_FILE32_TO_CHECK=$PVSW_ROOT/bin/psregsvr
        ZEN_FILE64_TO_CHECK=$PVSW_ROOT/bin/psregsvr64

        # First rename the bundled libstdc++ libs in case $PVSW_ROOT is
        # already in the path so the ldd check below will locate the system
        # library instead of the Zen-provided library (or not find it at all).
        if [ -f $PVSW_ROOT/lib/libstdc++.so.6 ]; then
            mv $PVSW_ROOT/lib/libstdc++.so.6 $PVSW_ROOT/lib/libstdc++.so.6.zen
        fi
        if [ -f $PVSW_ROOT/lib64/libstdc++.so.6 ]; then
            mv $PVSW_ROOT/lib64/libstdc++.so.6 $PVSW_ROOT/lib64/libstdc++.so.6.zen
        fi

        # Now locate 32-bit system library path from file dependencies for $ZEN_FILE32_TO_CHECK.
        if [ -f $ZEN_FILE32_TO_CHECK ]; then
            echo "Obtaining libstdc++ dependencies from 32-bit $ZEN_FILE32_TO_CHECK"
            SYS_INSTALLED_LIBSTDC32_PATH=`ldd $ZEN_FILE32_TO_CHECK | grep libstdc++.so.6 | cut -d " " -f 3`
            if [ -f "$SYS_INSTALLED_LIBSTDC32_PATH" ]; then
                echo "Checking for required version info in $SYS_INSTALLED_LIBSTDC32_PATH"
                FOUND_ZEN_LIBSTDC32_DEP=`readelf -V $SYS_INSTALLED_LIBSTDC32_PATH | grep GLIBCXX_$ZEN_REQUIRED_LIBSTDC_VER`
                if [ "X$FOUND_ZEN_LIBSTDC32_DEP" = "X" ]; then
                    echo "Required 32-bit version support not found, Zen will use 32-bit libstdc++.so.6 from install."
                    # Revert previous rename of libraries bundled with Zen install.
                    if [ -f $PVSW_ROOT/lib/libstdc++.so.6.zen ]; then
                        # Revert previous rename of libraries bundled with Zen install.
                        mv $PVSW_ROOT/lib/libstdc++.so.6.zen $PVSW_ROOT/lib/libstdc++.so.6
                    fi
                else
                    echo "Required 32-bit version support found, Zen will use $SYS_INSTALLED_LIBSTDC32_PATH"
                    # Must also remove previously renamed file or it will get a simlink created
                    # for it later in this script.
                    if [ -f $PVSW_ROOT/lib/libstdc++.so.6.zen ]; then
                        rm -f $PVSW_ROOT/lib/libstdc++.so.6.zen
                    fi
                fi
            else
                echo "Required 32-bit version support not found, Zen will use 32-bit libstdc++.so.6 from install."
                # Revert previous rename of libraries bundled with Zen install.
                if [ -f $PVSW_ROOT/lib/libstdc++.so.6.zen ]; then
                    # Revert previous rename of libraries bundled with Zen install.
                    mv $PVSW_ROOT/lib/libstdc++.so.6.zen $PVSW_ROOT/lib/libstdc++.so.6
                fi
            fi
        else
            echo "The 32-bit $ZEN_FILE32_TO_CHECK is not present in the Zen install package, skipping 32-bit libstdc++ dependency check."
        fi

        # Now locate 64-bit system library path from file dependencies for $ZEN_FILE64_TO_CHECK.
        if [ -f $ZEN_FILE64_TO_CHECK ]; then
            echo "Obtaining libstdc++ dependencies from 64-bit $ZEN_FILE64_TO_CHECK"
            SYS_INSTALLED_LIBSTDC64_PATH=`ldd $ZEN_FILE64_TO_CHECK | grep libstdc++.so.6 | cut -d " " -f 3`
            if [ -f "$SYS_INSTALLED_LIBSTDC64_PATH" ]; then
                echo "Checking for required version info in $SYS_INSTALLED_LIBSTDC64_PATH"
                FOUND_ZEN_LIBSTDC64_DEP=`readelf -V $SYS_INSTALLED_LIBSTDC64_PATH | grep GLIBCXX_$ZEN_REQUIRED_LIBSTDC_VER`
                if [ "X$FOUND_ZEN_LIBSTDC64_DEP" = "X" ]; then
                    echo "Required 64-bit version support not found, Zen will use 64-bit libstdc++.so.6 from install."
                    # Revert previous rename of libraries bundled with Zen install.
                    if [ -f $PVSW_ROOT/lib64/libstdc++.so.6.zen ]; then
                        # Revert previous rename of libraries bundled with Zen install.
                        mv $PVSW_ROOT/lib64/libstdc++.so.6.zen $PVSW_ROOT/lib64/libstdc++.so.6
                    fi
                else
                    echo "Required 64-bit version support found, Zen will use $SYS_INSTALLED_LIBSTDC64_PATH"
                    # Must also remove previously renamed file or it will get a simlink created
                    # for it later in this script.
                    if [ -f $PVSW_ROOT/lib64/libstdc++.so.6.zen ]; then
                        rm -f $PVSW_ROOT/lib64/libstdc++.so.6.zen
                    fi
                fi
            else
                echo "Required 64-bit version support not found, Zen will use 64-bit libstdc++.so.6 from install."
                # Revert previous rename of libraries bundled with Zen install.
                if [ -f $PVSW_ROOT/lib64/libstdc++.so.6.zen ]; then
                    # Revert previous rename of libraries bundled with Zen install.
                    mv $PVSW_ROOT/lib64/libstdc++.so.6.zen $PVSW_ROOT/lib64/libstdc++.so.6
                fi
            fi
        else
            echo "The 64-bit $ZEN_FILE64_TO_CHECK is not present in the Zen install, skipping 64-bit dependency check."
        fi
    fi
fi
echo

PATH=$PVSW_ROOT/bin:$PATH

if [ $(uname -s) = "Darwin" ]
then
    DYLD_LIBRARY_PATH=$PVSW_ROOT/bin:$PVSW_ROOT/lib:$PVSW_ROOT/lib64:$DYLD_LIBRARY_PATH
    export DYLD_LIBRARY_PATH
    SO_EXTENSION="dylib"
    SU_ARGS="-l"
else
    LD_LIBRARY_PATH=$PVSW_ROOT/bin:$PVSW_ROOT/lib:$PVSW_ROOT/lib64:$LD_LIBRARY_PATH
    export LD_LIBRARY_PATH
    SO_EXTENSION="so"
    SU_ARGS=
fi

export PVSW_ROOT
USERNAME=psql
GROUPNAME=pvsw
USERSHL=/bin/bash
USERSHLPROFILE1=.bash_profile
USERSHLPROFILE2=.bashrc
USERSHLPROFILE3=.cshrc

echo "Updating user $USERNAME environment..."
# Update the library path in the zen-svc profile files to include 32-bit lib folder path.
if [ $(uname -s) = "Darwin" ]
then
    HOMEDIR=/Users/"$USERNAME"
else
    HOMEDIR=`grep "^$USERNAME:" /etc/passwd | cut -d: -f6`
fi

PROFILE1=$HOMEDIR/$USERSHLPROFILE1
PROFILE2=$HOMEDIR/$USERSHLPROFILE2
PROFILE3=$HOMEDIR/$USERSHLPROFILE3

if [ $(uname -s) = "Darwin" ]
then
    echo export DYLD_LIBRARY_PATH=\$PVSW_ROOT/lib:\$DYLD_LIBRARY_PATH >> $PROFILE1
    echo export DYLD_LIBRARY_PATH=\$PVSW_ROOT/lib:\$DYLD_LIBRARY_PATH >> $PROFILE2
    echo setenv DYLD_LIBRARY_PATH \$PVSW_ROOT/lib:\$DYLD_LIBRARY_PATH >> $PROFILE3
else
    echo export LD_LIBRARY_PATH=\$PVSW_ROOT/lib:\$LD_LIBRARY_PATH >> $PROFILE1
    echo export LD_LIBRARY_PATH=\$PVSW_ROOT/lib:\$LD_LIBRARY_PATH >> $PROFILE2
    echo setenv LD_LIBRARY_PATH=\$PVSW_ROOT/lib:\$LD_LIBRARY_PATH >> $PROFILE3
fi
echo

echo "+++++ Setting up symbolic links for libraries..."

# These used to be in the RPM spec file, but we moved them
# here for tarball installs.
rm -f $PVSW_ROOT/lib/libbtrvif.${SO_EXTENSION}

LDCONFIG=/sbin/ldconfig
if [ ! -x $LDCONFIG ] ; then
    LDCONFIG=/usr/sbin/ldconfig
fi

if [ ! $(uname -s) = "Darwin" ]
then
    # Use ldconfig to create links to major versions
    # selinux: ldconfig runs in its own domain and may not have
    # permissions to write to tty.
    $LDCONFIG -n $PVSW_ROOT/lib 2>&1 | cat

    MIF_FILE=`ls -1 $PVSW_ROOT/lib/libpsqlmif.${SO_EXTENSION}.?? 2> /dev/null | tail -1`
    PSQL_VER=`echo $MIF_FILE | sed -e 's/.*[.]\(.*\)/\1/'`
    PS_VER=$PSQL_VER

    dso_symlink () {
        dso=$1
        ver=$2
        if [ -f $PVSW_ROOT/lib/$dso.$ver ]; then
            (cd $PVSW_ROOT/lib; ln -sf $dso.$ver $dso )
        fi
    }
else
    MIF_FILE=`ls -1 $PVSW_ROOT/lib/libpsqlmif.${SO_EXTENSION}.[0-9]* 2> /dev/null | tail -1`
    PSQL_VER=`echo $MIF_FILE | sed -e 's/^.*\.'"${SO_EXTENSION}"'\.\([0-9]*\)\..*$/\1/'`
    PS_VER=$PSQL_VER

    dso_symlink () {
      dso=$1
      ver=$2
      if [ -f $PVSW_ROOT/lib/$dso.$ver.[0-9]*.* ]; then
          (cd $PVSW_ROOT/lib; ln -sf $dso.$ver.[0-9]*.* $dso.$ver ; ln -sf $dso.$ver $dso )
      fi
    }

    dso_symlink libclientrb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libcsi100.${SO_EXTENSION} $PSQL_VER
    dso_symlink libdbcsipxy.${SO_EXTENSION} $PSQL_VER
    dso_symlink libdcm100.${SO_EXTENSION} $PSQL_VER
    dso_symlink libenginelm.${SO_EXTENSION} $PSQL_VER
    dso_symlink libexp010.${SO_EXTENSION} $PSQL_VER
    dso_symlink liblegacylm.${SO_EXTENSION} $PSQL_VER
    dso_symlink liblicmgrrb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libmkc3.${SO_EXTENSION} $PSQL_VER
    dso_symlink libmkderb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libodbccr.${SO_EXTENSION} 1
    dso_symlink libpceurop.${SO_EXTENSION} $PS_VER
    dso_symlink libpctlgrb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libpscl.${SO_EXTENSION} $PS_VER
    dso_symlink libpscore.${SO_EXTENSION} $PS_VER
    dso_symlink libpscp932.${SO_EXTENSION} $PS_VER
    dso_symlink libpsdom.${SO_EXTENSION} $PS_VER
    dso_symlink libpseucjp.${SO_EXTENSION} $PS_VER
    dso_symlink libpssax.${SO_EXTENSION} $PS_VER
    dso_symlink libpsutilrb.${SO_EXTENSION} $PS_VER
    dso_symlink libsrderb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libupiapirb.${SO_EXTENSION} $PSQL_VER
    dso_symlink libxlate.${SO_EXTENSION} $PSQL_VER
fi

dso_symlink libodbc.${SO_EXTENSION} 1
dso_symlink libpsqldti.${SO_EXTENSION} $PSQL_VER
dso_symlink libclientlm.${SO_EXTENSION} $PSQL_VER

dso_symlink libels.${SO_EXTENSION} '*.*.*.*'

dso_symlink libbtrieveC.${SO_EXTENSION} $PSQL_VER
dso_symlink libbtrieveCpp.${SO_EXTENSION} $PSQL_VER
dso_symlink libpsqlmif.${SO_EXTENSION} $PSQL_VER
dso_symlink libpsqlnsl.${SO_EXTENSION} $PSQL_VER
dso_symlink libpsqlcsm.${SO_EXTENSION} $PSQL_VER
dso_symlink libpsqlcsp.${SO_EXTENSION} $PSQL_VER
dso_symlink libodbcci.${SO_EXTENSION} $PSQL_VER
dso_symlink libcsi100.${SO_EXTENSION} $PSQL_VER

cd $PVSW_ROOT/lib 
ln -sf pam_pvsw.${SO_EXTENSION}.1.0 pam_pvsw.${SO_EXTENSION}
ln -sf libpsqlmif.${SO_EXTENSION} libbtrvif.${SO_EXTENSION}
ln -sf libodbcci.${SO_EXTENSION} odbcci.${SO_EXTENSION} 
cd $PVSW_ROOT 
echo

SELINUX_ENABLED=/usr/sbin/selinuxenabled

if [ -x $SELINUX_ENABLED ] && $SELINUX_ENABLED; then
    /sbin/restorecon -R $PVSW_ROOT
    if [ -d "$PVSW_ROOT"/lib ]
    then
    /usr/bin/chcon -t textrel_shlib_t "$PVSW_ROOT"/lib/*.so "$PVSW_ROOT"/lib/*.so.*
    fi
fi


echo "+++++ Registering the PCOM libraries..."

# Sym links must already exist
dso_register () {
    dso=$1
    ver=$2
    if [ -f $PVSW_ROOT/lib/$dso.$ver ]; then
        $PVSW_ROOT/bin/psregsvr $PVSW_ROOT/lib/$dso.$ver
        if [ $? -ne 0 ]; then
            echo ERROR: PCOM registration failed for $dso.$ver.
    	    exit 1;
    	fi
    fi
}
dso_register libpctlgrb.${SO_EXTENSION} $PSQL_VER
dso_register libpceurop.${SO_EXTENSION} $PS_VER
dso_register libpssax.${SO_EXTENSION} $PS_VER
dso_register libpsdom.${SO_EXTENSION} $PS_VER
dso_register libpsutilrb.${SO_EXTENSION} $PS_VER
dso_register libclientrb.${SO_EXTENSION} $PSQL_VER
dso_register libupiapirb.${SO_EXTENSION} $PSQL_VER
dso_register libcsi100.${SO_EXTENSION} $PSQL_VER
dso_register libmkc3.${SO_EXTENSION} $PSQL_VER
dso_register libpvmsgrb.${SO_EXTENSION} $PSQL_VER
dso_register libpsqlcsm.${SO_EXTENSION} $PSQL_VER
dso_register libpsqlcsp.${SO_EXTENSION} $PSQL_VER
dso_register libdbcsipxy.${SO_EXTENSION} $PSQL_VER
dso_register libpsqlmpm.${SO_EXTENSION} $PSQL_VER
dso_register libpscp932.${SO_EXTENSION} $PS_VER
dso_register libsrderb.${SO_EXTENSION} $PSQL_VER


echo "+++++ Making user $USERNAME owner of all files in $PVSW_ROOT/lib and $PVSW_ROOT/bin..."
chown -f $USERNAME:$GROUPNAME $PVSW_ROOT
chown -Rf $USERNAME:$GROUPNAME $PVSW_ROOT/lib
chown -Rf $USERNAME:$GROUPNAME $PVSW_ROOT/bin
chown -f $USERNAME:$GROUPNAME $PVSW_ROOT/etc

echo Install has completed.

echo
